﻿using Syncfusion.ListView.XForms;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Xamarin.Forms;
using System.Reflection;
using System.Windows.Input;
using Xamarin.Forms.Internals;
using Syncfusion.DataSource;

namespace Grouping
{
    public class ContactsViewModel : INotifyPropertyChanged
    {
        #region Fields

        private Contacts tappedItem;

        private SfListView listview;

        private Command<Object> expandOrCollapseCommand;

        private Command<Object> ascendingOrDescendingCommand;

        private ListViewSortOptions sortingOptions;

        #endregion

        #region Properties

        /// <summary>
        /// Gets or sets the value which indicates that instance of <see cref="SfListView"/>.
        /// </summary>
        public SfListView ListView
        {
            get { return listview; }
            set { listview = value; }
        }

        public ListViewSortOptions SortingOptions
        {
            get
            {
                return sortingOptions;
            }
            set
            {
                sortingOptions = value;
                RaisedOnPropertyChanged("SortingOptions");
            }
        }
        public ObservableCollection<Contacts> ContactsInfo { get; set; }

        public Command<Object> ExpandOrCollapseCommand
        {
            get { return expandOrCollapseCommand; }
            set { expandOrCollapseCommand = value; }
        }

        public Command<Object> AscendingOrDescendingCommand
        {
            get { return ascendingOrDescendingCommand; }
            set { ascendingOrDescendingCommand = value; }
        }

        #endregion

        #region Constructor

        public ContactsViewModel()
        {
            ContactsInfo = new ObservableCollection<Contacts>();
            Random r = new Random();
            for (int i = 0; i < 10; i++)
            {
                var contact = new Contacts();
                contact.ContactName = CustomerNames[i];
                contact.ContactNumber = r.Next(720, 799).ToString() + " - " + r.Next(3010, 3999).ToString();
                contact.Designation = CustomerDesignation[r.Next(0, CustomerDesignation.Count() - 1)];
                contact.Color = colors[r.Next(0, colors.Count() - 1)];
                var assembly = typeof(GroupingPage).GetTypeInfo().Assembly;
                contact.ContactImage = ImageSource.FromResource("Grouping.Images.Image" + r.Next(0, 28) + ".png", assembly);
                ContactsInfo.Add(contact);
            }

            ExpandOrCollapseCommand = new Command<Object>(ExpandOrCollapseCommandTapped);
            AscendingOrDescendingCommand = new Command<Object>(AscendingOrDescendingCommandTapped);
        }

        #endregion

        #region Private Methods

        private void ExpandOrCollapseCommandTapped(Object e)
        {
            tappedItem = e as Contacts;
            if (this.ContactsInfo.Contains(tappedItem) && tappedItem.IsVisible)
            {
                this.ContactsInfo.FirstOrDefault(x => x.ContactName == tappedItem.ContactName).IsVisible = false;
                this.ContactsInfo.FirstOrDefault(x => x.ContactName == tappedItem.ContactName).ExpandOrCollapse = "+";
            }
            else if (this.ContactsInfo.Contains(tappedItem))
            {
                this.ContactsInfo.FirstOrDefault(x => x.ContactName == tappedItem.ContactName).IsVisible = true;
                this.ContactsInfo.FirstOrDefault(x => x.ContactName == tappedItem.ContactName).ExpandOrCollapse = "-";
            }

            listview.ForceUpdateItemSize();
            listview.RefreshView();
        }

        private void AscendingOrDescendingCommandTapped(Object e)
        {
            var sort = ((e as Grid).Children[0] as Label).Text;

            var image = (e as Grid).Children[1] as Image;
            var imagename = image.Source.ToString();
            if (imagename == "File: GroupCollapse.png")
                image.Source = "GroupExpand.png";
            else
                image.Source = "GroupCollapse.png";


            if (this.SortingOptions == ListViewSortOptions.Descending)
                this.SortingOptions = ListViewSortOptions.Ascending;
            else if (this.SortingOptions == ListViewSortOptions.Ascending)
                this.SortingOptions = ListViewSortOptions.Descending;

            
            listview.DataSource.SortDescriptors.Clear();

            if (sort == "Contact Name")
            {
                listview.DataSource.SortDescriptors.Add(new SortDescriptor()
                {
                    PropertyName = "ContactName",
                    Direction = this.SortingOptions == ListViewSortOptions.Ascending ? ListSortDirection.Ascending : ListSortDirection.Descending
                });
            }
            else if(sort == "Contact Number")
            {
                listview.DataSource.SortDescriptors.Add(new SortDescriptor()
                {
                    PropertyName = "ContactNumber",
                    Direction = this.SortingOptions == ListViewSortOptions.Ascending ? ListSortDirection.Ascending : ListSortDirection.Descending
                });
            }
            else if (sort == "Designation")
            {
                listview.DataSource.SortDescriptors.Add(new SortDescriptor()
                {
                    PropertyName = "Designation",
                    Direction = this.SortingOptions == ListViewSortOptions.Ascending ? ListSortDirection.Ascending : ListSortDirection.Descending
                });
            }


        }

        #endregion

            #region Fields
        public string[] CustomerDesignation = new string[]
        {
            "Engineer",
            "Tester",
            "Manager",
            "Coordinator"
        };
        public string[] colors = new string[]
        {
            "Orange",
            "Blue",
            "Green",
            "Red",
            "Gray",
        };

        public string[] CustomerNames = new string[] {
            "Kyle",
            "Gina",
            "Irene",
            "Katie",
            "Michael",
            "Oscar",
            "Ralph",
            "Torrey",
            "William",
            "Bill",
            "Daniel",
            "Frank",
            "Brenda",
            "Jace   ",
            "Brandon",
            "Josiah ",
            "Hudson ",
            "Nathaniel",
            "Bryson ",
            "Ryder  ",
            "Justin ",
            "Bryce  ",
        };

        #endregion

        #region INotifyPropertyChanged

        public event PropertyChangedEventHandler PropertyChanged;
        public void RaisedOnPropertyChanged(string _PropertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(_PropertyName));
            }
        }

        #endregion
        public enum ListViewSortOptions
        {
            Ascending,
            Descending
        }
    }
}
